use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007004
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
Abstract
Anderson
Atlassian
Category
Change
Changelog
Class
Component
Factory
Issue
Item
Iterator
JIRA
LinkType
Packy
Project
Query
REST
ScriptRunner
Sprint
Status
TODO
TimeTracking
Transition
Transitions
Type
User
Version
Worklog
aggregateprogress
aggregatetimeestimate
aggregatetimeoriginalestimate
assigneeType
avatar
avatarUrls
completeDate
displayName
duedate
emailAddress
endDate
fieldtype
fixVersions
fromString
genericized
iconUrl
isAssigneeTypeValid
issueTypes
issuelinks
issuetype
jira
jql
lastViewed
lib
maxResults
originalEstimate
originalEstimateSeconds
packy
parentkey
projectId
rapidViewId
remainingEstimate
remainingEstimateSeconds
resolutiondate
sprintlist
startDate
subtaskIssueTypes
timeSpent
timeSpentSeconds
timeestimate
timeoriginalestimate
timespent
timetracking
toString
updateAuthor
worklog
workratio
